%AVERAGESAME averages images within a stack of equal exposure time
%            and writes out the average image
%
%  SYNOPSIS out= averagesame(nameformat,inputdir,expt,bin,n)
%
%  PARAMETERS
%   expt:      integration (exposure) time
%   bin:       binning of the images
%   n:         number of images with the same exposure times
%  inputdir:   directory where images are located
%              dir0=0 selects directory through user interface
%  nameformat: 'dark' or 'white'
%   NOTE :     outliers from the images should be first excluded with function
%              remout

% (C) Copyright 2009
%  Quantitative Imaging Group      Leiden University Medical Center
%  Faculty of Applied Sciences      Department of Molecular Cell Biology
%  Delft University of Technology  Section Electron Microscopy
%  Lorentzweg 1                    2300 RC Leiden
%  2628 CJ Delft
%  The Netherlands
%
% Milos Vulovic & Bernd Rieger

function out= averagesame(varargin)

d = struct('menu','TEM CCD',...
           'display','Average background image',...
           'inparams',struct('name',       {'s0', 'dir0', 'expt', 'binning', 'n'},...
                             'description',{'Name Format','Input directory','Exposure Time','Binning', 'Number of Images'},...
                             'type',       {'option','indir','array','array','array'},...
                             'dim_check',  {0,0,-1,-1,0},...
                             'range_check',{{'dark','white'},'','R+','N+','N+'},...
                             'required',   {1,1,1,0,0},...
                             'default',    {'dark','',0,1,10}...
                            ),...
           'outparams',struct('name',{'out'},...
                              'description',{'Output image'},...
                              'type',{'image'}...
                              )...
          );
if nargin == 1
   s = varargin{1};
   if ischar(s) & strcmp(s,'DIP_GetParamList')
      out = d;
      return
   end
end
try
   [s0, dir0, expt, binning, n] = getparams(d,varargin{:});
catch
   if ~isempty(paramerror)
      error(paramerror)
   else
      error(firsterr)
   end
end









if dir0==0
    dir0=uigetdir('/home', 'Select directory where the reference images are' );
end

switch s0
    case 'dark'
        s0 = 'dezinger_Bin%d_Exp%05.2f_dark%02d';
        q=0;
    case 'white'
        s0 = 'dezinger_Bin%d_Exp%05.2f_white%02d';
        q=1;
    otherwise
        error('Unknown filename format.')
end

nr=1:n;
bin=binning;
cnt=0;
fn=[];
for n1=nr;
    cnt=cnt+1;
    s1=sprintf(s0,bin,expt,n1);
    fn{cnt}=[dir0 filesep s1];
    fprintf('file %s\n',fn{cnt});
    
end
tmpim = readim(fn{1});
sz=size(tmpim);

all1=newim(sz(1) ,sz(2) ,n);
all2=newim(sz(1) ,sz(2) ,n);

if q==0
    % read in dark referencies
    
    for j=1:n
        s0d=sprintf(s0,bin,expt,j);
        s1d=[dir0 filesep s0d];
        fprintf('Reading image %s\n',s0d);
        Id=readim(s1d);
        all2(:,:,j-1)=Id;
    end
    
    % average dark
    all2m=mean(all2,[],3);
    Ibg=squeeze(all2m);
    out=Ibg;
    
    % Write out the average background image for given int time and binning
    p0='av_dark_bin%d_Exp%05.2f';
    p2=[dir0 filesep sprintf(p0,bin,expt)];
    writeim(dip_image(Ibg,'sfloat'),p2,'tiff','yes');
    fprintf('Writing out %s\n',p2);
else
    
    %reading of white references
    for j=1:n
        s0g=sprintf(s0,bin,expt,j);
        s1=[dir0 filesep s0g];
        fprintf('Reading image %s\n',s0g);
        I=readim(s1);
        all1(:,:,j-1)=I;
    end
    %average white
    all1m=mean(all1,[],3);
    Iwhite=squeeze(all1m);
    out=Iwhite;
    
    % Write out the average white image for given exp time and binning
    k1=sprintf('av_white_bin%d_Exp%05.2f',bin,expt);
    k2=[dir0 filesep k1];
    writeim(dip_image(Iwhite,'sfloat'),k2,'tiff','yes');
    fprintf('Writing out %s\n',k2);
    
end

