% %MTF_NOISE calculates the MTF from corrected uniformly illuminated image(s) 
% %           and plots the normalized MTF
% %
% % SYNOPSIS:
% %  out = mtf_noise(in,dirradlib)
% %  
% % PARAMETERS:
% %       in: image or stack of corrected images of uniform illumination     
% % dirradlib  directory where radius_lib directory is. This directory contains
% %            some gaussian profiles to facilitate angular averaging. After dowloading it 
% %            could be found in the same directory as provided functions (pwd). If
% %            these files are not present in the library (due to the different choose of paramters for Gaussian rings),
% %            the function will generate them 
% %
% %    NOTES:  The recomended mean intensity in the image is
% %            1/3 - 1/2 of the whole range. Stack can be made by 
% %            function readtimeseries.m
 
% % (C) Copyright 2009
% %  Quantitative Imaging Group      Leiden University Medical Center
% %  Faculty of Applied Sciences     Department of Molecular Cell Biology
% %  Delft University of Technology  Section Electron Microscopy
% %  Lorentzweg 1                    2300 RC Leiden
% %  2628 CJ Delft
% %  The Netherlands
% %
% % Milos Vulovic & Bernd Rieger      
% 
function out=mtf_noise(varargin)

d = struct('menu','TEM CCD',...
           'display','MTF estimation (noise method)',...
           'inparams',struct('name',       {'im', 'dirradlib' },...
                             'description',{'Input image (or stack)', 'Radius_libdir'},...
                             'type',       {'image','indir'},...
                             'dim_check',  {[],0},...
                             'range_check',{[], ''},...
                             'required',   {1,1},...
                             'default',    {'im', ''}...
                            ),...
           'outparams',struct('name',{'out'},...
                              'description',{'MTF'},...
                              'type',{'array'}...
                              )...
          );
if nargin == 1
   s = varargin{1};
   if ischar(s) & strcmp(s,'DIP_GetParamList')
      out = d;
      return
   end
end
try
   [im, dirradlib0] = getparams(d,varargin{:});
catch
   if ~isempty(paramerror)
      error(paramerror)
   else
      error(firsterr)
   end
end

prvi=8;
offset=5;
sz=size(im);
if ndims(im)==2;n=1; im=cat(3,im,im);
else n=sz(3);end

%tmpim = ex_slice(im,0);
tmpim = squeeze(im(:,:,0));
sz0=size(tmpim);

if sz0(1)>= 512 
    rangex=1:512; 
    rangey=rangex;
    radiusstep = 2;
elseif sz0(1) <512 && sz0(1) >= 256
    rangex=1:256; 
    rangey=rangex;
    radiusstep = 1;
else
    error(' Image must be larger than 256x256')
end

%camsize =min(size(a,1));
%processsize = camsize;%if I want to process the whole image
b_c = 1; %binning of an acquired image
b_c2 =1; %additional binning in the real space for fastening the calculation
b_f  = 1; %additional binning in the frequency space for make it faster
ellipR = 1; %parameters of ellipse ratio between long and short axis
ellipang = 0; %angular direction of the main axis of the ellipse.
processsize=length(rangex)/(b_c2*b_f);
%%% building the averaging 'gaussian cone-cube'
radiusstep = 1;
sigmanul = 2;
sigmaR =0.9;

dirradlib=[dirradlib0 filesep 'radius_lib'];
radfile = strcat([dirradlib filesep 'astigR'],num2str(ellipR),'astigang',num2str(ellipang),'_rad_step',num2str(radiusstep),'_max',...
    num2str(processsize),'_sig',num2str(sigmaR),'to',num2str(sigmanul),'.mat');
if exist (radfile,'file') 
    load(radfile)
else
     [radius_range, radius] = mk_gauss_cone_cube(radiusstep, processsize, ellipR, ellipang, sigmanul, sigmaR);
    save( radfile , 'radius_range', 'radius' )
end



for kk=1:n
    fprintf('processing image number %2d\n',kk);
%     sl=ex_slice(im,kk-1);
    sl=squeeze(im(:,:,kk-1));
    a=cut(sl, length (rangex));
    sz=size(a);

    %%% loading the file and calculating PSD of the reduced intensity
    if b_c2 > 1
        a = rebin(a,b_c2)/b_c2;
    end
    b = ft(sqrt(a)-mean(sqrt(a)));
    b = abs(b);
    if b_f > 1
        b = rebin(b,b_f)/b_f;
    end
    radialaverage0 = radav( b, radius );
    radialaverage1(kk,:)=double(radialaverage0);
end

radialaverage=mean(radialaverage1,1);
pik=max(radialaverage(1:prvi));
angav=radialaverage/pik;
m=max(radius_range);

figure
plot(radius_range/m, angav,'-b');
cent=size(2*radius_range/sz(1));
out=angav;
niqa=angav(round(cent(2)/2));
title ('MTF noise method');
xlabel('Nyquist')
ylabel('MTF')
   
s=sprintf('0.5 Nq %0.2g\n',niqa);
h1=axis;
text (0.65*h1(2),0.6*h1(4),s,'FontSize',10)
